Package: wyz.code.offensiveProgramming
Type: Package
Title: Wizardry Code Offensive Programming
Version: 1.1.24
Author: Fabien Gelineau <neonira@gmail.com>
Maintainer: Fabien Gelineau <neonira@gmail.com>
Description: Allows to turn standard R code into offensive programming code. 
    Provides code instrumentation to ease this change and tools to assist and 
    accelerate code production and tuning while using offensive programming code 
    technics. 
    Should improve code robustness and quality. Function calls can be easily 
    verified on-demand or in batch mode to assess parameter types and length 
    conformities. 
    Should improve coders productivity as offensive programming reduces the code
    size due to reduced number of controls all along the call chain. 
    Should speed up processing as many checks will be reduced to one single check.
Encoding: UTF-8
License: GPL-3
Depends: R (>= 4.0)
Imports: methods, data.table (>= 1.11.8), tidyr, stringr (>= 1.4.0), R6
        (>= 2.4.0), crayon
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 6.1.1
VignetteBuilder: knitr
URL: https://neonira.github.io/offensiveProgrammingBook_v1.2.2/
NeedsCompilation: no
Packaged: 2023-09-25 06:33:54 UTC; fabien
Repository: CRAN
Date/Publication: 2023-09-25 07:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 13:31:39 UTC; unix
